<?php

/**
 * @file
 * Functions supporting the operation of Encrypted Files
 */

/**
 * Generates two keys for encrypting files.
 *
 * @return array
 *   Array with both keys
 */
function _encrypted_files_generate_keys() {
  $keys = array();
  // Generate both keys.
  for ($i = 0; $i < 2; $i++) {
    $random_value = microtime(TRUE) . mt_rand(10000, mt_getrandmax());
    // Use bcrypt if available.
    if (defined('CRYPT_BLOWFISH') && CRYPT_BLOWFISH) {
      $bcrypt_salt = '$2a$10$H.1bnj0AKNT.Ulj1dM3M12';
      $keys[] = crypt($random_value, $bcrypt_salt);
    }
    // Otherwise, use SHA.
    else {
      $keys[] = hash('sha256', $random_value);
    }
  }

  return $keys;
}

/**
 * Encrypts the second key with the first key.
 *
 * @param string $key1
 *   Key to use for encryption
 * @param string $key2
 *   Key to encrypt
 *
 * @return string
 *   The encrypted second key
 */
function _encrypted_files_encrypt_key($key1, $key2) {
  return aes_encrypt($key2, TRUE, $key1);
}

/**
 * Decrypts the second key with the first key.
 *
 * @param string $key1
 *   Key to use for decryption
 * @param string $key2
 *   Key to decrypt
 *
 * @return string
 *   The unencrypted second key
 */
function _encrypted_files_decrypt_key($key1, $key2) {
  return aes_decrypt($key2, TRUE, $key1);
}

/**
 * Loads both keys, one from file system and one from database.
 *
 * @return array|bool
 *   Array with both keys or FALSE if loading either key failed
 */
function _encrypted_files_load_keys() {
  $keys = array();
  // Load first key from file system.
  $key1 = file_get_contents(variable_get('file_private_path',
    'sites/default/files/private') .
    '/' . ENCRYPTED_FILES_KEY_FILE);
  if ($key1 === FALSE) {
    watchdog('encrypted_files', 'Unable to load key file from ' .
      'the private file system.', array(), WATCHDOG_ERROR);
    return FALSE;
  }
  $keys[] = $key1;
  // Load second key from database.
  $key2 = variable_get(ENCRYPTED_FILES_KEY_VARIABLE, FALSE);
  if ($key2 === FALSE) {
    watchdog('encrypted_files', 'Unable to load key file from the database.',
      array(), WATCHDOG_ERROR);
    return FALSE;
  }
  $keys[] = $key2;

  return $keys;
}

/**
 * Saves both keys, one to file system and one to database.
 *
 * @param array $keys
 *   Array of keys to save
 *
 * @return bool
 *   TRUE if save successful, FALSE if not
 */
function _encrypted_files_save_keys($keys) {
  // Save first key to file system.
  $written = file_put_contents(variable_get('file_private_path',
    'sites/default/files/private') .
    '/' . ENCRYPTED_FILES_KEY_FILE, $keys[0]);
  if (!$written) {
    watchdog('encrypted_files', 'Unable to save key file ' .
      'to the private file system.', array(), WATCHDOG_ERROR);
    return FALSE;
  }
  // Encrypt and save second key to database.
  $key2 = _encrypted_files_encrypt_key($keys[0], $keys[1]);
  variable_set(ENCRYPTED_FILES_KEY_VARIABLE, $key2);

  return TRUE;
}
